@@ -1,16 +1,22 @@
-"""Local HDF5 Backend Implementation, Identifier: ``HDF5_00``
+"""Local HDF5 Backend Implementation, Identifier: ``HDF5_01``
 
 Backend Identifiers
 ===================
 
 *  Backend: ``0``
-*  Version: ``0``
-*  Format Code: ``00``
-*  Canonical Name: ``HDF5_00``
+*  Version: ``1``
+*  Format Code: ``01``
+*  Canonical Name: ``HDF5_01``
 
 Storage Method
 ==============
 
+*  This module is meant to handle larger datasets which are of fixed size. IO
+   and significant compression optimization is achieved by storing arrays at
+   their appropriate top level index in the same shape they naturally assume
+   and chunking over the entire subarray domain making up a sample (rather than
+   having to subdivide chunks when the sample could be variably shaped.)
+
 *  Data is written to specific subarray indexes inside an HDF5 "dataset" in a
    single HDF5 File.
 
@@ -21,13 +27,64 @@ Storage Method
 
    *  ``dtype: {schema_dtype}``; ie ``np.float32`` or ``np.uint8``
 
-   *  ``shape: (COLLECTION_SIZE, *{schema_shape})``; ie ``(500, 10)`` or ``(500,
-      4, 3)``. The first index in the dataset is referred to as a ``collection
-      index``.
+   *  ``shape: (COLLECTION_SIZE, *{schema_shape})``; ie ``(500, 10, 10)`` or
+      ``(500, 512, 512, 320)``. The first index in the dataset is referred to as a
+      ``collection index``.
 
 *  Compression Filters, Chunking Configuration/Options are applied globally for
    all ``datasets`` in a file at dataset creation time.
 
+*  On read and write of all samples the xxhash64_hexdigest is calculated for
+   the raw array bytes. This is to ensure that all data in == data out of the
+   hdf5 files. That way even if a file is manually edited (bypassing fletcher32
+   filter check) we have a quick way to tell that things are not as they should
+   be.
+
+Compression Options
+===================
+
+Accepts dictionary containing keys
+
+*  ``backend`` == ``"01"``
+*  ``complib``
+*  ``complevel``
+*  ``shuffle``
+
+Blosc-HDF5
+
+*  ``complib`` valid values:
+
+   *  ``'blosc:blosclz'``,
+   *  ``'blosc:lz4'``,
+   *  ``'blosc:lz4hc'``,
+   *  ``'blosc:zlib'``,
+   *  ``'blosc:zstd'``
+
+*  ``complevel`` valid values: [0, 9] where 0 is "no compression" and 9 is
+   "most compression"
+
+*  ``shuffle`` valid values:
+
+   *  ``None``
+   *  ``'none'``
+   *  ``'byte'``
+   *  ``'bit'``
+
+
+LZF Filter
+
+*  ``'complib' == 'lzf'``
+*  ``'shuffle'`` one of ``[False, None, 'none', True, 'byte']``
+*  ``'complevel'`` one of ``[False, None, 'none']``
+
+GZip Filter
+
+*  ``'complib' == 'gzip'``
+*  ``'shuffle'`` one of ``[False, None, 'none', True, 'byte']``
+*  ``complevel`` valid values: [0, 9] where 0 is "no compression" and 9 is
+   "most compression"
+
+
 Record Format
 =============
 
@@ -36,8 +93,9 @@ Fields Recorded for Each Array
 
 *  Format Code
 *  File UID
+*  xxhash64_hexdigest (ie. checksum)
 *  Dataset Number (``0:COLLECTION_COUNT`` dataset selection)
-*  Collection Index (``0:COLLECTION_SIZE`` dataset subarray selection)
+*  Dataset Index (``0:COLLECTION_SIZE`` dataset subarray selection)
 *  Subarray Shape
 
 Separators used
@@ -53,26 +111,31 @@ Examples
 
 1)  Adding the first piece of data to a file:
 
-    *  Array shape (Subarray Shape): (10)
-    *  File UID: "2HvGf9"
-    *  Dataset Number: "0"
-    *  Collection Index: 0
+    *  Array shape (Subarray Shape): (10, 10)
+    *  File UID: "rlUK3C"
+    *  xxhash64_hexdigest: 8067007c0f05c359
+    *  Dataset Number: 16
+    *  Collection Index: 105
 
-   ``Record Data => "00:2HvGf9$0 0*10"``
+    ``Record Data => "01:rlUK3C$8067007c0f05c359$16 105*10 10"``
 
 1)  Adding to a piece of data to a the middle of a file:
 
     *  Array shape (Subarray Shape): (20, 2, 3)
-    *  File UID: "WzUtdu"
+    *  File UID: "rlUK3C"
+    *  xxhash64_hexdigest: b89f873d3d153a9c
     *  Dataset Number: "3"
     *  Collection Index: 199
 
-    ``Record Data => "00:WzUtdu$3 199*20 2 3"``
+    ``Record Data => "01:rlUK3C$b89f873d3d153a9c$8 199*20 2 3"``
 
 
 Technical Notes
 ===============
 
+*  The majority of methods not directly related to "chunking" and the "raw data
+   chunk cache" are either identical to HDF5_00, or only slightly modified.
+
 *  Files are read only after initial creation/writes. Only a write-enabled
    checkout can open a HDF5 file in ``"w"`` or ``"a"`` mode, and writer
    checkouts create new files on every checkout, and make no attempt to fill in
@@ -89,19 +152,85 @@ Technical Notes
    via custom python ``pickle`` serialization/reduction logic which is
    implemented by the high level ``pickle`` reduction ``__set_state__()``,
    ``__get_state__()`` class methods.
+
+*  An optimization is performed in order to increase the read / write
+   performance of fixed size datasets. Due to the way that we initialize an
+   entire HDF5 file with all datasets pre-created (to the size of the fixed
+   subarray shape), and the fact we absolutely know the size / shape /
+   access-pattern of the arrays, inefficient IO due to wasted chunk processing
+   is not a concern. It is far more efficient for us to completely blow off the
+   metadata chunk cache, and chunk each subarray as a single large item item.
+
+   This method of processing tends to have a number of significant effects as
+   compared to chunked storage methods:
+
+      1. **Compression rations improve** (by a non-trivial factor). This is
+         simply due to the fact that a larger amount of raw data is being passed
+         into the compressor at a time. While the exact improvement seen is
+         highly dependent on both the data size and compressor used, there
+         should be no case where compressing the full tensor uses more disk
+         space then chunking the tensor, compressing each chunk individually,
+         and then saving each chunk to disk.
+
+      2. **Read performance improves** (so long as a suitable compressor /
+         option set was chosen). Instead of issuing (potentially) many read
+         requests - one for each chunk - to the storage hardware, signifiantly
+         few IOPS are used to retrieve the entire set of compressed raw data
+         from disk. Fewer IOPS means much less time waiting on the hard disk.
+         Moreso, only a single decompression step is needed to reconstruct
+         the numeric array, completly decoupling performance from HDF5's
+         ability to parallelize internal filter pipeline operations.
+
+         Additionally, since the entire requested chunk is retrieved in a
+         single decompression pipeline run, there is no need for the HDF5 core
+         to initialize an intermediate buffer which holds data chunks as each
+         decompression operation completes. Futher, by preinitializing an empty
+         ``numpy.ndarray`` container and using the low level HDF5
+         ``read_direct`` method, the decompressed data buffer is passes
+         directly into the returned ``ndarray.__array_interface__.data``
+         field with no intermediate copy or processing steps.
+
+      3. **Shuffle filters are favored.**. With much more data to work with in
+         a single compression operation, the use of "byte shuffle" filters in
+         the compressor spec has been seen to both markedly decrease read time
+         and increase compression ratios. Shuffling can significantly reduce
+         disk space required to store some piece of data on disk, further
+         reducing the time spent waiting on hard disk IO while incuring a
+         negligible cost to decompression speed.
+
+   Taking all of these effects into account, there can be up to an order of
+   magnitude increase in read performance as compared to the subarray chunking
+   strategy employed by the ``HDF5_00`` backend.
+
+*  Like all other backends at the time of writing, only 'C' ordered arrays
+   are accepted by this method.
 """
-import math
 import os
 import re
 import time
+import logging
 from collections import ChainMap
 from os.path import join as pjoin
 from os.path import splitext as psplitext
 from functools import partial
-from typing import MutableMapping, NamedTuple, Tuple, Optional, Union, Callable, Pattern
+from typing import (
+    MutableMapping, NamedTuple, Tuple, Optional, Union, Callable, Pattern)
 
-import h5py
 import numpy as np
+import h5py
+try:
+    # hdf5plugin warns if a filter is already loaded. we temporarily surpress
+    # that here, then reset the logger level to it's initial version.
+    _logger = logging.getLogger('hdf5plugin')
+    _initialLevel = _logger.getEffectiveLevel()
+    _logger.setLevel(logging.ERROR)
+    import hdf5plugin
+    assert 'blosc' in hdf5plugin.FILTERS
+except (ImportError, ModuleNotFoundError):  # pragma: no cover
+    pass
+finally:
+    _logger.setLevel(_initialLevel)
+from xxhash import xxh64_hexdigest
 
 from .. import __version__
 from .. import constants as c
@@ -114,54 +243,42 @@ set_blosc_nthreads()
 
 
 # contents of a single hdf5 file
-COLLECTION_SIZE = 250
+COLLECTION_SIZE = 100
 COLLECTION_COUNT = 100
-
-# chunking options for compression schemes
-CHUNK_MAX_NBYTES = 200_000  # < 256 KB to fit in L2 CPU Cache
-CHUNK_MAX_RDCC_NBYTES = 100_000_000
+CHUNK_MAX_RDCC_NBYTES = 500_000_000
 CHUNK_RDCC_W0 = 0.75
 
-# filter definition and backup selection if not available.
-filter_opts = {
-    'default': {
-        'shuffle': 'bit',
-        'complib': 'blosc:lz4',
-        'complevel': 7,
-        'fletcher32': True},
-    'backup': {
-        'shuffle': True,
-        'complib': 'lzf',
-        'complevel': None,
-        'fletcher32': True},
-}
-hdf5BloscAvail = h5py.h5z.filter_avail(32001)
-HDF5_FILTER = filter_opts['default'] if hdf5BloscAvail else filter_opts['backup']
-
 
 # -------------------------------- Parser Implementation ----------------------
 
 
-_FmtCode = '00'
+_FmtCode = '01'
 # match and remove the following characters: '['   ']'   '('   ')'   ','
 _ShapeFmtRE: Pattern = re.compile('[,\(\)\[\]]')
 # split up a formated parsed string into unique fields
-_SplitDecoderRE: Pattern = re.compile(fr'[\{c.SEP_KEY}\{c.SEP_HSH}\{c.SEP_SLC}]')
+_patern = fr'\{c.SEP_KEY}\{c.SEP_HSH}\{c.SEP_SLC}'
+_SplitDecoderRE: Pattern = re.compile(fr'[{_patern}]')
 
 
-HDF5_00_DataHashSpec = NamedTuple('HDF5_00_DataHashSpec',
-                                  [('backend', str), ('uid', str),
-                                   ('dataset', str), ('dataset_idx', int),
-                                   ('shape', Tuple[int])])
+HDF5_01_DataHashSpec = NamedTuple('HDF5_01_DataHashSpec', [
+    ('backend', str),
+    ('uid', str),
+    ('checksum', str),
+    ('dataset', str),
+    ('dataset_idx', int),
+    ('shape', Tuple[int])])
 
 
-def hdf5_00_encode(uid: str, dataset: str, dataset_idx: int, shape: Tuple[int]) -> bytes:
+def hdf5_01_encode(uid: str, checksum: str, dataset: str, dataset_idx: int,
+                   shape: Tuple[int]) -> bytes:
     """converts the hdf5 data has spec to an appropriate db value
 
     Parameters
     ----------
     uid : str
         the file name prefix which the data is written to.
+    checksum : int
+        xxhash_64.hex_digest checksum of the data bytes in numpy array form.
     dataset : str
         collection (ie. hdf5 dataset) name to find this data piece.
     dataset_idx : int
@@ -176,15 +293,14 @@ def hdf5_00_encode(uid: str, dataset: str, dataset_idx: int, shape: Tuple[int])
     bytes
         hash data db value recording all input specifications.
     """
-    out_str = f'{_FmtCode}{c.SEP_KEY}{uid}'\
-              f'{c.SEP_HSH}'\
-              f'{dataset}{c.SEP_LST}{dataset_idx}'\
-              f'{c.SEP_SLC}'\
+    out_str = f'{_FmtCode}{c.SEP_KEY}'\
+              f'{uid}{c.SEP_HSH}{checksum}{c.SEP_HSH}'\
+              f'{dataset}{c.SEP_LST}{dataset_idx}{c.SEP_SLC}'\
               f'{_ShapeFmtRE.sub("", str(shape))}'
     return out_str.encode()
 
 
-def hdf5_00_decode(db_val: bytes) -> HDF5_00_DataHashSpec:
+def hdf5_01_decode(db_val: bytes) -> HDF5_01_DataHashSpec:
     """converts an hdf5 data hash db val into an hdf5 data python spec.
 
     Parameters
@@ -194,19 +310,20 @@ def hdf5_00_decode(db_val: bytes) -> HDF5_00_DataHashSpec:
 
     Returns
     -------
-    HDF5_00_DataHashSpec
+    HDF5_01_DataHashSpec
         hdf5 data hash specification containing `backend`, `schema`,
         `instance`, `dataset`, `dataset_idx`, `shape`
     """
     db_str = db_val.decode()
-    _, uid, dataset_vs, shape_vs = _SplitDecoderRE.split(db_str)
+    _, uid, checksum, dataset_vs, shape_vs = _SplitDecoderRE.split(db_str)
     dataset, dataset_idx = dataset_vs.split(c.SEP_LST)
     # if the data is of empty shape -> shape_vs = '' str.split() default value
     # of none means split according to any whitespace, and discard empty strings
     # from the result. So long as c.SEP_LST = ' ' this will work
     shape = tuple(int(x) for x in shape_vs.split())
-    raw_val = HDF5_00_DataHashSpec(backend=_FmtCode,
+    raw_val = HDF5_01_DataHashSpec(backend=_FmtCode,
                                    uid=uid,
+                                   checksum=checksum,
                                    dataset=dataset,
                                    dataset_idx=int(dataset_idx),
                                    shape=shape)
@@ -216,10 +333,10 @@ def hdf5_00_decode(db_val: bytes) -> HDF5_00_DataHashSpec:
 # ------------------------- Accessor Object -----------------------------------
 
 
-HDF5_00_MapTypes = MutableMapping[str, Union[h5py.File, Callable[[], h5py.File]]]
+HDF5_01_MapTypes = MutableMapping[str, Union[h5py.File, Callable[[], h5py.File]]]
 
 
-class HDF5_00_FileHandles(object):
+class HDF5_01_FileHandles(object):
     """Manage HDF5 file handles.
 
     When in SWMR-write mode, no more than a single file handle can be in the
@@ -231,10 +348,11 @@ class HDF5_00_FileHandles(object):
         self.path: os.PathLike = repo_path
         self.schema_shape: tuple = schema_shape
         self.schema_dtype: np.dtype = schema_dtype
+        self._dflt_backend_opts: Optional[dict] = None
 
-        self.rFp: HDF5_00_MapTypes = {}
-        self.wFp: HDF5_00_MapTypes = {}
-        self.Fp: HDF5_00_MapTypes = ChainMap(self.rFp, self.wFp)
+        self.rFp: HDF5_01_MapTypes = {}
+        self.wFp: HDF5_01_MapTypes = {}
+        self.Fp: HDF5_01_MapTypes = ChainMap(self.rFp, self.wFp)
 
         self.mode: Optional[str] = None
         self.hIdx: Optional[int] = None
@@ -273,14 +391,26 @@ class HDF5_00_FileHandles(object):
         del state['Fp']
         return state
 
-    def __setstate__(self, state: dict) -> None:
+    def __setstate__(self, state: dict) -> None:  # pragma: no cover
         """ensure multiprocess operations can pickle relevant data.
         """
         self.__dict__.update(state)
         self.rFp = {}
         self.wFp = {}
         self.Fp = ChainMap(self.rFp, self.wFp)
-        self.open(self.mode)
+        self.open(mode=self.mode)
+
+    @property
+    def backend_opts(self):
+        return self._dflt_backend_opts
+
+    @backend_opts.setter
+    def backend_opts(self, val):
+        if self.mode == 'a':
+            self._dflt_backend_opts = val
+            return
+        else:
+            raise AttributeError(f"can't set property in read only mode")
 
     def open(self, mode: str, *, remote_operation: bool = False):
         """Open an hdf5 file handle in the Handler Singleton
@@ -289,7 +419,7 @@ class HDF5_00_FileHandles(object):
         ----------
         mode : str
             one of `r` or `a` for read only / read-write.
-        repote_operation : optional, kwarg only, bool
+        remote_operation : optional, kwarg only, bool
             if this hdf5 data is being created from a remote fetch operation, then
             we don't open any files for reading, and only open files for writing
             which exist in the remote data dir. (default is false, which means that
@@ -379,7 +509,7 @@ class HDF5_00_FileHandles(object):
         os.rmdir(process_dir)
 
     @staticmethod
-    def _dataset_opts(complib: str, complevel: int, shuffle: Union[bool, str], fletcher32: bool) -> dict:
+    def _dataset_opts(complib: str, complevel: int, shuffle: Union[bool, str]) -> dict:
         """specify compression options for the hdf5 dataset.
 
         .. seealso:: :function:`_blosc_opts`
@@ -395,78 +525,77 @@ class HDF5_00_FileHandles(object):
         ----------
         complib : str
             the compression lib to use, one of ['lzf', 'gzip', 'blosc:blosclz',
-            'blosc:lz4', 'blosc:lz4hc', 'blosc:snappy', 'blosc:zlib', 'blosc:zstd']
+            'blosc:lz4', 'blosc:lz4hc', 'blosc:zlib', 'blosc:zstd']
         complevel : int
             compression level to specify (accepts values [0, 9] for all except 'lzf'
             where no complevel is accepted)
         shuffle : bool
-            if True, enable byte shuffle filter, if blosc compression, pass through
-            'bits' is accepted as well.
-        fletcher32 : bool
-            enable fletcher32 checksum validation of data integrity, (defaults to
-            True, which is enabled)
+            if True or `byte`, enable byte shuffle filter, if blosc
+            compression, pass through 'bits' is accepted as well. False, or
+            None indicates no shuffle should be applied.
         """
+        # ---- blosc hdf5 plugin filters ----
+        _blosc_shuffle = {
+            None: 0,
+            'none': 0,
+            'byte': 1,
+            'bit': 2}
+        _blosc_compression = {
+            'blosc:blosclz': 0,
+            'blosc:lz4': 1,
+            'blosc:lz4hc': 2,
+            # Not built 'snappy': 3,
+            'blosc:zlib': 4,
+            'blosc:zstd': 5}
+        _blosc_complevel = {
+            **{i: i for i in range(10)},
+            None: 9,
+            'none': 9}
+
+        # ---- h5py built in filters ----
+        _lzf_gzip_shuffle = {
+            None: False,
+            False: False,
+            'none': False,
+            True: True,
+            'byte': True}
+        _lzf_complevel = {
+            False: None,
+            None: None,
+            'none': None}
+        _gzip_complevel = {
+            **{i: i for i in range(10)},
+            None: 4,
+            'none': 4}
+
         if complib.startswith('blosc'):
-            shuffle = 2 if shuffle == 'bit' else 1 if shuffle else 0
-            compressors = ['blosclz', 'lz4', 'lz4hc', 'snappy', 'zlib', 'zstd']
-            complib = ['blosc:' + c for c in compressors].index(complib)
             args = {
                 'compression': 32001,
-                'compression_opts': (0, 0, 0, 0, complevel, shuffle, complib),
-                'fletcher32': fletcher32,
-            }
-            if shuffle:
-                args['shuffle'] = False
-        else:
+                'compression_opts': (
+                    0, 0, 0, 0,
+                    _blosc_complevel[complevel],
+                    _blosc_shuffle[shuffle],
+                    _blosc_compression[complib]),
+                'shuffle': False}
+        elif complib == 'lzf':
             args = {
-                'shuffle': shuffle,
+                'shuffle': _lzf_gzip_shuffle[shuffle],
                 'compression': complib,
-                'compression_opts': None if complib == 'lzf' else complevel,
-                'fletcher32': fletcher32,
-            }
+                'compression_opts': _lzf_complevel[complevel]}
+        elif complib == 'gzip':
+            args = {
+                'shuffle': _lzf_gzip_shuffle[shuffle],
+                'compression': complib,
+                'compression_opts': _gzip_complevel[complevel]}
+        elif complib in (None, False, 'none'):
+            args = {
+                'shuffle': False,
+                'compression': None,
+                'compression_opts': None}
+        else:
+            raise ValueError(f'unknown value for opt arg `complib`: {complib}')
         return args
 
-    @staticmethod
-    def _chunk_opts(sample_array: np.ndarray, max_chunk_nbytes: int) -> Tuple[list, int]:
-        """Determine the chunk shape so each array chunk fits into configured nbytes.
-
-        Currently the chunk nbytes are not user configurable. Instead the constant
-        `HDF5_MAX_CHUNK_NBYTES` is sued to determine when to split.
-
-        Parameters
-        ----------
-        sample_array : `np.array`
-            Sample array whose shape and dtype should be used as the basis of the
-            chunk shape determination
-        max_chunk_nbytes : int
-            how many bytes the array chunks should be limited to.
-
-        Returns
-        -------
-        list
-            list of ints of length == rank of `sample_array` specifying chunk sizes
-            to split `sample_array` into nbytes
-        int
-            nbytes which the chunk will fit in. Will be <= `HDF5_MAX_CHUNK_NBYTES`
-        """
-        chunk_nbytes = sample_array.nbytes
-        chunk_shape = list(sample_array.shape)
-        shape_rank = len(chunk_shape)
-        chunk_idx = 0
-
-        while chunk_nbytes > max_chunk_nbytes:
-            if chunk_idx >= shape_rank:
-                chunk_idx = 0
-            rank_dim = chunk_shape[chunk_idx]
-            if rank_dim <= 2:
-                chunk_idx += 1
-                continue
-            chunk_shape[chunk_idx] = math.floor(rank_dim / 2)
-            chunk_nbytes = np.zeros(shape=chunk_shape, dtype=sample_array.dtype).nbytes
-            chunk_idx += 1
-
-        return (chunk_shape, chunk_nbytes)
-
     def _create_schema(self, *, remote_operation: bool = False):
         """stores the shape and dtype as the schema of a arrayset.
 
@@ -504,18 +633,12 @@ class HDF5_00_FileHandles(object):
 
         # -------------------- Chunk & RDCC Vals ------------------------------
 
-        sample_array = np.zeros(self.schema_shape, dtype=self.schema_dtype)
-        chunk_shape, chunk_nbytes = __class__._chunk_opts(
-            sample_array=sample_array,
-            max_chunk_nbytes=CHUNK_MAX_NBYTES)
-
-        rdcc_nbytes_val = sample_array.nbytes * COLLECTION_SIZE
-        if rdcc_nbytes_val < CHUNK_MAX_NBYTES:
-            rdcc_nbytes_val = CHUNK_MAX_NBYTES
-        elif rdcc_nbytes_val > CHUNK_MAX_RDCC_NBYTES:
+        chunk_shape = self.schema_shape
+        chunk_nbytes = np.dtype(self.schema_dtype).itemsize * np.prod(chunk_shape)
+        rdcc_nbytes_val = chunk_nbytes * COLLECTION_SIZE
+        if rdcc_nbytes_val >= CHUNK_MAX_RDCC_NBYTES:
             rdcc_nbytes_val = CHUNK_MAX_RDCC_NBYTES
-
-        rdcc_nslots_guess = math.ceil(rdcc_nbytes_val / chunk_nbytes) * 100
+        rdcc_nslots_guess = np.math.ceil(rdcc_nbytes_val / chunk_nbytes) * 100
         rdcc_nslots_prime_val = find_next_prime(rdcc_nslots_guess)
 
         # ---------------------------- File Creation --------------------------
@@ -542,21 +665,20 @@ class HDF5_00_FileHandles(object):
 
         # ----------------------- Dataset Creation ----------------------------
 
-        optKwargs = self._dataset_opts(**HDF5_FILTER)
+        optKwargs = self._dataset_opts(**self._dflt_backend_opts)
         for dset_num in range(COLLECTION_COUNT):
             self.wFp[uid].create_dataset(
                 f'/{dset_num}',
-                shape=(COLLECTION_SIZE, *sample_array.shape),
-                dtype=sample_array.dtype,
-                maxshape=(COLLECTION_SIZE, *sample_array.shape),
+                shape=(COLLECTION_SIZE, *chunk_shape),
+                dtype=self.schema_dtype,
                 chunks=(1, *chunk_shape),
                 **optKwargs)
 
         # ---------------------- Attribute Config Vals ------------------------
 
         self.wFp[self.w_uid]['/'].attrs['HANGAR_VERSION'] = __version__
-        self.wFp[self.w_uid]['/'].attrs['schema_shape'] = sample_array.shape
-        self.wFp[self.w_uid]['/'].attrs['schema_dtype_num'] = sample_array.dtype.num
+        self.wFp[self.w_uid]['/'].attrs['schema_shape'] = self.schema_shape
+        self.wFp[self.w_uid]['/'].attrs['schema_dtype_num'] = np.dtype(self.schema_dtype).num
         self.wFp[self.w_uid]['/'].attrs['next_location'] = (0, 0)
         self.wFp[self.w_uid]['/'].attrs['collection_max_size'] = COLLECTION_SIZE
         self.wFp[self.w_uid]['/'].attrs['collection_total'] = COLLECTION_COUNT
@@ -564,9 +686,6 @@ class HDF5_00_FileHandles(object):
         self.wFp[self.w_uid]['/'].attrs['rdcc_nbytes'] = rdcc_nbytes_val
         self.wFp[self.w_uid]['/'].attrs['rdcc_w0'] = CHUNK_RDCC_W0
         self.wFp[self.w_uid]['/'].attrs['rdcc_nslots'] = rdcc_nslots_prime_val
-        self.wFp[self.w_uid]['/'].attrs['shuffle'] = HDF5_FILTER['shuffle']
-        self.wFp[self.w_uid]['/'].attrs['complib'] = HDF5_FILTER['complib']
-        self.wFp[self.w_uid]['/'].attrs['fletcher32'] = HDF5_FILTER['fletcher32']
         self.wFp[self.w_uid]['/'].attrs['chunk_shape'] = chunk_shape
         if optKwargs['compression_opts'] is not None:
             self.wFp[self.w_uid]['/'].attrs['compression_opts'] = optKwargs['compression_opts']
@@ -579,12 +698,12 @@ class HDF5_00_FileHandles(object):
         except ValueError:
             assert self.wFp[self.w_uid].swmr_mode is True
 
-    def read_data(self, hashVal: HDF5_00_DataHashSpec) -> np.ndarray:
+    def read_data(self, hashVal: HDF5_01_DataHashSpec) -> np.ndarray:
         """Read data from an hdf5 file handle at the specified locations
 
         Parameters
         ----------
-        hashVal : HDF5_00_DataHashSpec
+        hashVal : HDF5_01_DataHashSpec
             record specification parsed from its serialized store val in lmdb.
 
         Returns
@@ -592,14 +711,13 @@ class HDF5_00_FileHandles(object):
         np.array
             requested data.
         """
-        dsetIdx = int(hashVal.dataset_idx)
         dsetCol = f'/{hashVal.dataset}'
-
-        srcSlc = (self.slcExpr[dsetIdx], *(self.slcExpr[0:x] for x in hashVal.shape))
+        # if len(hashVal.shape) > 0:
+        srcSlc = (self.slcExpr[int(hashVal.dataset_idx)], ...)
         destSlc = None
 
         if self.schema_dtype is not None:
-            destArr = np.empty((hashVal.shape), self.schema_dtype)
+            destArr = np.empty(hashVal.shape, self.schema_dtype)
             try:
                 self.Fp[hashVal.uid][dsetCol].read_direct(destArr, srcSlc, destSlc)
             except TypeError:
@@ -627,6 +745,13 @@ class HDF5_00_FileHandles(object):
                     destArr = self.Fp[hashVal.uid][dsetCol][srcSlc]
                 else:
                     raise
+
+        if xxh64_hexdigest(destArr) != hashVal.checksum:
+            # try casting to check if dtype does not match for all zeros case
+            destArr = destArr.astype(np.typeDict[self.Fp[hashVal.uid]['/'].attrs['schema_dtype_num']])
+            if xxh64_hexdigest(destArr) != hashVal.checksum:
+                raise RuntimeError(
+                    f'DATA CORRUPTION Checksum {xxh64_hexdigest(destArr)} != recorded {hashVal}')
         return destArr
 
     def write_data(self, array: np.ndarray, *, remote_operation: bool = False) -> bytes:
@@ -648,6 +773,7 @@ class HDF5_00_FileHandles(object):
             string identifying the collection dataset and collection dim-0 index
             which the array can be accessed at.
         """
+        checksum = xxh64_hexdigest(array)
         if self.w_uid in self.wFp:
             self.hIdx += 1
             if self.hIdx >= self.hMaxSize:
@@ -662,11 +788,9 @@ class HDF5_00_FileHandles(object):
         else:
             self._create_schema(remote_operation=remote_operation)
 
-        srcSlc = None
-        destSlc = (self.slcExpr[self.hIdx], *(self.slcExpr[0:x] for x in array.shape))
-        self.wFp[self.w_uid][f'/{self.hNextPath}'].write_direct(array, srcSlc, destSlc)
-
-        hashVal = hdf5_00_encode(uid=self.w_uid,
+        self.wFp[self.w_uid][f'/{self.hNextPath}'][self.hIdx] = array
+        hashVal = hdf5_01_encode(uid=self.w_uid,
+                                 checksum=checksum,
                                  dataset=self.hNextPath,
                                  dataset_idx=self.hIdx,
                                  shape=array.shape)
