@@ -6,6 +6,10 @@ SEP_CMT = ' << '
 SEP_SLC = "*"
 SEP_HSH = '$'
 
+CMT_KV_JOIN_KEY = SEP_LST.encode()
+CMT_DIGEST_JOIN_KEY = ''
+CMT_REC_JOIN_KEY = SEP_HSH.encode()
+
 K_INT = f'#'
 K_BRANCH = f'branch{SEP_KEY}'
 K_HEAD = 'head'
@@ -30,8 +34,8 @@ DIR_DATA_REMOTE = 'remote_data'
 
 # configuration file names:
 
-CONFIG_USER_NAME = 'config_user.yml'
-CONFIG_SERVER_NAME = 'config_server.yml'
+CONFIG_USER_NAME = 'config_user.ini'
+CONFIG_SERVER_NAME = 'config_server.ini'
 
 # LMDB database names and settings.
 
@@ -40,7 +44,7 @@ LMDB_SETTINGS = {
     'meminit': False,
     'subdir': False,
     'lock': False,
-    'max_spare_txns': 2,
+    'max_spare_txns': 4,
 }
 
 LMDB_REF_NAME = 'ref.lmdb'
@@ -53,4 +57,4 @@ LMDB_STAGE_HASH_NAME = 'stage_hash.lmdb'
 
 # readme file
 
-README_FILE_NAME = 'README.txt'
\ No newline at end of file
+README_FILE_NAME = 'README.txt'
