@@ -1,5 +1,4 @@
 import os
-import hashlib
 from typing import Optional, Union, Iterator, Tuple, Dict
 
 import lmdb
@@ -7,6 +6,7 @@ import lmdb
 from .context import TxnRegister
 from .records import parsing
 from .records.queries import RecordQuery
+from .records.hashmachine import metadata_hash_digest
 from .utils import is_suitable_user_key, is_ascii
 
 
@@ -61,7 +61,6 @@ class MetadataReader(object):
         self._mode = mode
         self._path = repo_pth
         self._is_conman: bool = False
-
         self._labelenv: lmdb.Environment = labelenv
         self._labelTxn: Optional[lmdb.Transaction] = None
         self._TxnRegister = TxnRegister()
@@ -231,7 +230,6 @@ class MetadataReader(object):
             tmpconman = not self._is_conman
             if tmpconman:
                 self.__enter__()
-
             metaVal = self._labelTxn.get(self._mspecs[key])
             meta_val = parsing.hash_meta_raw_val_from_db_val(metaVal)
         except KeyError:
@@ -239,7 +237,6 @@ class MetadataReader(object):
         finally:
             if tmpconman:
                 self.__exit__()
-
         return meta_val
 
 
@@ -274,7 +271,6 @@ class MetadataWriter(MetadataReader):
         """
 
         super().__init__(*args, **kwargs)
-
         self._dataenv: lmdb.Environment = kwargs['dataenv']
         self._dataTxn: Optional[lmdb.Transaction] = None
 
@@ -331,7 +327,7 @@ class MetadataWriter(MetadataReader):
         Parameters
         ----------
         key : Union[str, int]
-            Name of the metadata piece, alphanumeric ascii characters only
+            Name of the metadata piece, alphanumeric ascii characters only.
         value : string
             Metadata value to store in the repository, any length of valid
             ascii characters.
@@ -345,7 +341,8 @@ class MetadataWriter(MetadataReader):
         Raises
         ------
         ValueError
-            If the `key` contains any whitespace or non alpha-numeric characters.
+            If the `key` contains any whitespace or non alpha-numeric
+            characters or is longer than 64 characters.
         ValueError
             If the `value` contains any non ascii characters.
         """
@@ -353,7 +350,8 @@ class MetadataWriter(MetadataReader):
             if not is_suitable_user_key(key):
                 raise ValueError(
                     f'Metadata key: {key} of type: {type(key)} invalid. Must be int '
-                    f'ascii string with only alpha-numeric / "." "_" "-" characters.')
+                    f'ascii string with only alpha-numeric / "." "_" "-" characters. '
+                    f'Must be <= 64 characters long.')
             elif not (isinstance(value, str) and is_ascii(value)):
                 raise ValueError(
                     f'Metadata Value: `{value}` not allowed. Must be ascii-only string')
@@ -365,11 +363,10 @@ class MetadataWriter(MetadataReader):
             if tmpconman:
                 self.__enter__()
 
-            val_hash = hashlib.blake2b(value.encode(), digest_size=20).hexdigest()
+            val_hash = metadata_hash_digest(value=value)
             hashKey = parsing.hash_meta_db_key_from_raw_key(val_hash)
             metaRecKey = parsing.metadata_record_db_key_from_raw_key(key)
             metaRecVal = parsing.metadata_record_db_val_from_raw_val(val_hash)
-
             # check if meta record already exists with same key
             existingMetaRecVal = self._dataTxn.get(metaRecKey, default=False)
             if existingMetaRecVal:
@@ -384,20 +381,12 @@ class MetadataWriter(MetadataReader):
                 hashVal = parsing.hash_meta_db_val_from_raw_val(value)
                 self._labelTxn.put(hashKey, hashVal)
 
-            # increment metadata record count
-            if existingMetaRecVal is False:
-                metaCountKey = parsing.metadata_count_db_key()
-                metaCountVal = self._dataTxn.get(metaCountKey, default='0'.encode())
-                meta_count = parsing.metadata_count_raw_val_from_db_val(metaCountVal) + 1
-                newMetaCountVal = parsing.metadata_count_db_val_from_raw_val(meta_count)
-                self._dataTxn.put(metaCountKey, newMetaCountVal)
             self._dataTxn.put(metaRecKey, metaRecVal)
             self._mspecs[key] = hashKey
 
         finally:
             if tmpconman:
                 self.__exit__()
-
         return key
 
     def remove(self, key: Union[str, int]) -> Union[str, int]:
@@ -416,9 +405,6 @@ class MetadataWriter(MetadataReader):
 
         Raises
         ------
-        ValueError
-            If the key provided is not string type and containing only
-            ascii-alphanumeric characters.
         KeyError
             If the checkout does not contain metadata with the provided key.
         """
@@ -426,34 +412,18 @@ class MetadataWriter(MetadataReader):
             tmpconman = not self._is_conman
             if tmpconman:
                 self.__enter__()
-
-            if not is_suitable_user_key(key):
-                msg = f'HANGAR VALUE ERROR:: metadata key: `{key}` not allowed. Must be str'\
-                      f'containing alpha-numeric or "." "_" "-" ascii characters (no whitespace).'
-                raise ValueError(msg)
+            if key not in self._mspecs:
+                raise KeyError(f'No metadata exists with key: {key}')
 
             metaRecKey = parsing.metadata_record_db_key_from_raw_key(key)
             delete_succeeded = self._dataTxn.delete(metaRecKey)
             if delete_succeeded is False:
-                msg = f'HANGAR KEY ERROR:: No metadata exists with key: {key}'
-                raise KeyError(msg)
+                raise KeyError(f'No metadata exists with key: {key}')
             del self._mspecs[key]
 
-            metaRecCountKey = parsing.metadata_count_db_key()
-            metaRecCountVal = self._dataTxn.get(metaRecCountKey)
-            meta_count = parsing.metadata_count_raw_val_from_db_val(metaRecCountVal)
-            meta_count -= 1
-            if meta_count == 0:
-                self._dataTxn.delete(metaRecCountKey)
-            else:
-                newMetaRecCountVal = parsing.metadata_count_db_val_from_raw_val(meta_count)
-                self._dataTxn.put(metaRecCountKey, newMetaRecCountVal)
-
         except (KeyError, ValueError) as e:
             raise e from None
-
         finally:
             if tmpconman:
                 self.__exit__()
-
         return key
