@@ -1,17 +1,17 @@
-import hashlib
 import os
 import tempfile
 import time
 from os.path import join as pjoin
 import shutil
+from contextlib import contextmanager
+import configparser
 
 import lmdb
-import yaml
 
 from . import heads, parsing
 from .. import constants as c
 from ..context import TxnRegister
-from .queries import RecordQuery
+from .parsing import DigestAndBytes
 from ..utils import symlink_rel
 
 
@@ -123,7 +123,7 @@ def get_commit_spec(refenv, commit_hash):
         raise ValueError(f'No commit exists with the hash: {commit_hash}')
 
     parentCommitSpec = parsing.commit_spec_raw_val_from_db_val(parentCommitSpecVal)
-    return parentCommitSpec
+    return parentCommitSpec.user_spec
 
 
 def get_commit_ancestors(refenv, commit_hash):
@@ -139,7 +139,7 @@ def get_commit_ancestors(refenv, commit_hash):
     Returns
     -------
     namedtuple
-        Namedtuple describing is_merge_commit, master_ancester, &
+        Namedtuple describing is_merge_commit, master_ancestor, &
         child_ancestor (in the even of merge commit)
 
     Raises
@@ -159,7 +159,7 @@ def get_commit_ancestors(refenv, commit_hash):
         raise ValueError(f'No commit exists with the hash: {commit_hash}')
 
     parentCommitAncestors = parsing.commit_parent_raw_val_from_db_val(parentCommitVal)
-    return parentCommitAncestors
+    return parentCommitAncestors.ancestor_spec
 
 
 def get_commit_ancestors_graph(refenv, starting_commit):
@@ -231,7 +231,7 @@ def get_commit_ref(refenv, commit_hash):
 
     Parameters
     ----------
-    refenv : lmdb.Envirionment`
+    refenv : lmdb.Environment`
         lmdb environment where the references are stored
     commit_hash : string
         hash of the commit to retrieve.
@@ -249,48 +249,47 @@ def get_commit_ref(refenv, commit_hash):
     """
     reftxn = TxnRegister().begin_reader_txn(refenv)
     try:
-        commitRefKey = parsing.commit_ref_db_key_from_raw_key(commit_hash)
-        commitRefVal = reftxn.get(commitRefKey, default=False)
+        cmtRefKey = parsing.commit_ref_db_key_from_raw_key(commit_hash)
+        cmtSpecKey = parsing.commit_spec_db_key_from_raw_key(commit_hash)
+        cmtParentKey = parsing.commit_parent_db_key_from_raw_key(commit_hash)
+
+        cmtRefVal = reftxn.get(cmtRefKey, default=False)
+        cmtSpecVal = reftxn.get(cmtSpecKey, default=False)
+        cmtParentVal = reftxn.get(cmtParentKey, default=False)
+    except lmdb.BadValsizeError:
+        raise ValueError(f'No commit exists with the hash: {commit_hash}')
     finally:
         TxnRegister().abort_reader_txn(refenv)
 
-    if commitRefVal is False:
+    if (cmtRefVal is False) or (cmtSpecVal is False) or (cmtParentVal is False):
         raise ValueError(f'No commit exists with the hash: {commit_hash}')
 
-    commitRefs = parsing.commit_ref_raw_val_from_db_val(commitRefVal)
-    return commitRefs
+    commitRefs = parsing.commit_ref_raw_val_from_db_val(cmtRefVal)
+    commitSpecs = parsing.commit_spec_raw_val_from_db_val(cmtSpecVal)
+    commitParent = parsing.commit_parent_raw_val_from_db_val(cmtParentVal)
 
+    calculatedDigest = parsing.cmt_final_digest(
+        parent_digest=commitParent.digest,
+        spec_digest=commitSpecs.digest,
+        refs_digest=commitRefs.digest)
 
-def get_commit_ref_contents(refenv, commit_hash):
-    """hackish way to convert lmdb from compressed structure to in memory without a new env.
+    if calculatedDigest != commit_hash:
+        raise IOError(
+            f'DATA CORRUPTION ERROR: on retrieval of stored references for '
+            f'commit_hash: {commit_hash} validation of commit record/contents '
+            f'integrity failed. Calculated digest: {calculatedDigest} != '
+            f'expected: {commit_hash}. Please alert the Hangar development team to '
+            f'this error if possible.')
 
-    .. todo:: Completly refactor this mess...
-
-    Parameters
-    ----------
-    refenv : lmdb.Environment
-        lmdb environment where the commit refs are stored
-    commit_hash : str
-        hash of the commit to get the contents of
-
-    Returns
-    -------
-    dict
-        nested dict
-    """
-    with tempfile.TemporaryDirectory() as tempD:
-        tmpDF = os.path.join(tempD, 'test.lmdb')
-        tmpDB = lmdb.open(path=tmpDF, **c.LMDB_SETTINGS)
-        unpack_commit_ref(refenv, tmpDB, commit_hash)
-        outDict = RecordQuery(tmpDB).all_records()
-        tmpDB.close()
-
-    return outDict
+    return commitRefs.db_kvs
 
 
 def unpack_commit_ref(refenv, cmtrefenv, commit_hash):
     """unpack a commit record ref into a new key/val db for reader checkouts.
 
+    This method also validates that the record data (parent, spec, and refs)
+    have not been corrupted on disk (ie)
+
     Parameters
     ----------
     refenv : lmdb.Environment
@@ -319,59 +318,86 @@ def unpack_commit_ref(refenv, cmtrefenv, commit_hash):
     return
 
 
+@contextmanager
+def tmp_cmt_env(refenv: lmdb.Environment, commit_hash: str):
+    """create temporary unpacked lmdb environment from compressed structure
+
+    Parameters
+    ----------
+    refenv : lmdb.Environment
+        lmdb environment where the commit refs are stored
+    commit_hash : str
+        hash of the commit to get the contents of
+
+    Returns
+    -------
+    lmdb.Environment
+        environment with all db contents from ``commit`` unpacked
+    """
+    tempD = tempfile.mkdtemp()
+    try:
+        tmpDF = os.path.join(tempD, 'test.lmdb')
+        tmpDB = lmdb.open(path=tmpDF, sync=False, writemap=True, **c.LMDB_SETTINGS)
+        unpack_commit_ref(refenv, tmpDB, commit_hash)
+        yield tmpDB
+    finally:
+        tmpDB.close()
+        shutil.rmtree(tempD)
+
+
 """
 Methods to write new commits
 ----------------------------
 
 The functions below act to:
-    - Reading and formating all record data from the staging area.
+    - Reading and formatting all record data from the staging area.
     - Determining the ancestor(s) of the new commit
-    - Specify commit details (message, time, commiter-info, etc.)
+    - Specify commit details (message, time, committer-info, etc.)
     - Coordinate record hashing
     - Write the commit record
     - Update the branch head to point to the new commit hash
 """
 
-# ---------------- Functions to format the writen values of a commit --------------------
+# ---------------- Functions to format the written values of a commit --------------------
 
 
 def _commit_ancestors(branchenv: lmdb.Environment,
                       *,
                       is_merge_commit: bool = False,
-                      master_branch_name: str = '',
-                      dev_branch_name: str = '') -> bytes:
+                      master_branch: str = '',
+                      dev_branch: str = '') -> DigestAndBytes:
     """Format the commit parent db value, finding HEAD commits automatically.
 
-    This method handles formating for both regular & merge commits through the
+    This method handles formatting for both regular & merge commits through the
     the keyword only arguments.
 
     Parameters
     ----------
     branchenv : lmdb.Environment
-        Lmdb envrionment where branch data is located. If not merge commit, head
+        Lmdb environment where branch data is located. If not merge commit, head
         branch and commit will be found.
     is_merge_commit : bool, optional
         If this is a merge commit or now, defaults to False
-    master_branch_name : str, optional
+    master_branch : str, optional
         If merge commit, the master branch name must be specified, and the
         branch HEAD commit hash will be determined automatically, defaults to ''
-    dev_branch_name : str, optional
+    dev_branch : str, optional
         If merge commit, the dev branch name must be specified, and the branch
         HEAD commit hash will be determined automatically, defaults to ''
 
     Returns
     -------
-    bytes
-        Commit parent db value formatted appropriately based on the repo state and
-        any specified arguments.
+    DigestAndBytes
+        Commit parent db value and digest of commit parent val formatted
+        appropriately based on the repo state and any specified arguments.
     """
     if not is_merge_commit:
         masterBranch = heads.get_staging_branch_head(branchenv)
         master_ancestor = heads.get_branch_head_commit(branchenv, masterBranch)
         dev_ancestor = ''
     else:
-        master_ancestor = heads.get_branch_head_commit(branchenv, master_branch_name)
-        dev_ancestor = heads.get_branch_head_commit(branchenv, dev_branch_name)
+        master_ancestor = heads.get_branch_head_commit(branchenv, master_branch)
+        dev_ancestor = heads.get_branch_head_commit(branchenv, dev_branch)
 
     commitParentVal = parsing.commit_parent_db_val_from_raw_val(
         master_ancestor=master_ancestor,
@@ -381,7 +407,7 @@ def _commit_ancestors(branchenv: lmdb.Environment,
     return commitParentVal
 
 
-def _commit_spec(message: str, user: str, email: str) -> bytes:
+def _commit_spec(message: str, user: str, email: str) -> DigestAndBytes:
     """Format the commit specification according to the supplied username and email.
 
     This method currently only acts as a pass through to the parsing options
@@ -392,24 +418,24 @@ def _commit_spec(message: str, user: str, email: str) -> bytes:
     message : string
         Commit message sent in by the user.
     user : str, optional
-        Name of the commiter
+        Name of the committer
     email : str, optional
         Email of the committer
 
     Returns
     -------
-    bytes
-        Formatted value for the specification field of the commit.
+    DigestAndBytes
+        Formatted value for the specification field of the commit and digest of
+        spec.
     """
-    commitSpecVal = parsing.commit_spec_db_val_from_raw_val(
-        commit_time=time.time(),
-        commit_message=message,
-        commit_user=user,
-        commit_email=email)
-    return commitSpecVal
+    spec_db = parsing.commit_spec_db_val_from_raw_val(commit_time=time.time(),
+                                                      commit_message=message,
+                                                      commit_user=user,
+                                                      commit_email=email)
+    return spec_db
 
 
-def _commit_ref(stageenv: lmdb.Environment) -> bytes:
+def _commit_ref(stageenv: lmdb.Environment) -> DigestAndBytes:
     """Query and format all staged data records, and format it for ref storage.
 
     Parameters
@@ -419,14 +445,16 @@ def _commit_ref(stageenv: lmdb.Environment) -> bytes:
 
     Returns
     -------
-    bytes
-        Serialized and compressed version of all staged record data.
-
+    DigestAndBytes
+        Serialized and compressed version of all staged record data along with
+        digest of commit refs.
     """
+    from .queries import RecordQuery  # needed to avoid cyclic import
+
     querys = RecordQuery(dataenv=stageenv)
     allRecords = tuple(querys._traverse_all_records())
-    commitRefVal = parsing.commit_ref_db_val_from_raw_val(allRecords)
-    return commitRefVal
+    res = parsing.commit_ref_db_val_from_raw_val(allRecords)
+    return res
 
 
 # -------------------- Format ref k/v pairs and write the commit to disk ----------------
@@ -442,7 +470,7 @@ def commit_records(message, branchenv, stageenv, refenv, repo_path,
     Parameters
     ----------
     message : string
-        Message the user accociates with what has been added, removed, or
+        Message the user associates with what has been added, removed, or
         changed in this commit. Must not be empty.
     branchenv : lmdb.Environment
         lmdb environment where branch records are stored.
@@ -463,28 +491,26 @@ def commit_records(message, branchenv, stageenv, refenv, repo_path,
     string
         Commit hash of the newly added commit
     """
-    commitParentVal = _commit_ancestors(branchenv=branchenv,
-                                         is_merge_commit=is_merge_commit,
-                                         master_branch_name=merge_master,
-                                         dev_branch_name=merge_dev)
+    cmtParent = _commit_ancestors(branchenv=branchenv,
+                                  is_merge_commit=is_merge_commit,
+                                  master_branch=merge_master,
+                                  dev_branch=merge_dev)
 
-    user_info_pth = pjoin(repo_path, 'config_user.yml')
-    with open(user_info_pth) as f:
-        user_info = yaml.safe_load(f.read()) or {}
+    user_info_pth = pjoin(repo_path, c.CONFIG_USER_NAME)
+    CFG = configparser.ConfigParser()
+    CFG.read(user_info_pth)
 
-    USER_NAME = user_info['name']
-    USER_EMAIL = user_info['email']
+    USER_NAME = CFG['USER']['name']
+    USER_EMAIL = CFG['USER']['email']
     if (USER_NAME is None) or (USER_EMAIL is None):
         raise RuntimeError(f'Username and Email are required. Please configure.')
 
-    commitSpecVal = _commit_spec(message=message, user=USER_NAME, email=USER_EMAIL)
-    commitRefVal = _commit_ref(stageenv=stageenv)
+    cmtSpec = _commit_spec(message=message, user=USER_NAME, email=USER_EMAIL)
+    cmtRefs = _commit_ref(stageenv=stageenv)
 
-    hasher = hashlib.blake2b(digest_size=20)
-    hasher.update(commitParentVal)
-    hasher.update(commitSpecVal)
-    hasher.update(commitRefVal)
-    commit_hash = hasher.hexdigest()
+    commit_hash = parsing.cmt_final_digest(parent_digest=cmtParent.digest,
+                                           spec_digest=cmtSpec.digest,
+                                           refs_digest=cmtRefs.digest)
 
     commitSpecKey = parsing.commit_spec_db_key_from_raw_key(commit_hash)
     commitParentKey = parsing.commit_parent_db_key_from_raw_key(commit_hash)
@@ -492,13 +518,13 @@ def commit_records(message, branchenv, stageenv, refenv, repo_path,
 
     reftxn = TxnRegister().begin_writer_txn(refenv)
     try:
-        reftxn.put(commitSpecKey, commitSpecVal, overwrite=False)
-        reftxn.put(commitParentKey, commitParentVal, overwrite=False)
-        reftxn.put(commitRefKey, commitRefVal, overwrite=False)
+        reftxn.put(commitSpecKey, cmtSpec.raw, overwrite=False)
+        reftxn.put(commitParentKey, cmtParent.raw, overwrite=False)
+        reftxn.put(commitRefKey, cmtRefs.raw, overwrite=False)
     finally:
         TxnRegister().commit_writer_txn(refenv)
 
-    # possible seperate function
+    # possible separate function
     move_process_data_to_store(repo_path)
     if is_merge_commit is False:
         headBranchName = heads.get_staging_branch_head(branchenv)
@@ -514,7 +540,7 @@ def commit_records(message, branchenv, stageenv, refenv, repo_path,
 
 
 def replace_staging_area_with_commit(refenv, stageenv, commit_hash):
-    """DANGER ZONE: Delete the stage db and replace it with a copy of a commit environent.
+    """DANGER ZONE: Delete the stage db and replace it with a copy of a commit environment.
 
     .. warning::
 
@@ -552,13 +578,13 @@ def replace_staging_area_with_refs(stageenv, sorted_content):
 
     Parameters
     ----------
-    stageenv : lmdb.Enviornment
+    stageenv : lmdb.Environment
         staging area db to replace all data in.
     sorted_content : iterable of tuple
-        iterable containing two-tuple of byte encoded record data to place in the
-        stageenv db. index 0 -> db key; index 1 -> db val, it is assumed that the
-        order of the tuples is lexigraphically sorted by index 0 values, if not,
-        this will result in unknown behavior.
+        iterable containing two-tuple of byte encoded record data to place in
+        the stageenv db. index 0 -> db key; index 1 -> db val, it is assumed
+        that the order of the tuples is lexicographically sorted by index 0
+        values, if not, this will result in unknown behavior.
     """
     stagetxn = TxnRegister().begin_writer_txn(stageenv)
     with stagetxn.cursor() as cursor:
@@ -584,20 +610,20 @@ def move_process_data_to_store(repo_path: str, *, remote_operation: bool = False
     In process writes never directly access files in the data directory.
     Instead, when the file is created is is symlinked to either the remote data
     or stage data directory. All access is handled through this intermediate
-    symlink in order to prevent any ability to overwpackedeven if
-    there are major errors in the hash records). Once the write operation
-    packedor staging, or completion of fetch for remote), this
-    method is called to move the symlinks from the write enabled directory to
-    the (read only, fully-committed) storage dir.
+    symlink in order to prevent any ability to overwrite (even if there are
+    major errors in the hash records). Once the write operation is packed in
+    the staging or remote area, this method is called to move the symlinks from
+    the write enabled directory to the (read only, fully-committed) storage
+    dir.
 
     Parameters
     ----------
     repo_path : str
         path to the repository on dir
     remote_operation : bool, optional
-        If this operation is occuring from a remote fetch operation. (the
-        default is False, which means that all changes will occur in the staging
-        area)
+        If this operation is occurring from a remote fetch operation. (the
+        default is False, which means that all changes will occur in the
+        staging area)
 
     """
     store_dir = pjoin(repo_path, c.DIR_DATA_STORE)
@@ -610,7 +636,15 @@ def move_process_data_to_store(repo_path: str, *, remote_operation: bool = False
     dirs_to_make, symlinks_to_make = [], []
     for root, dirs, files in os.walk(process_dir):
         for d in dirs:
-            dirs_to_make.append(os.path.relpath(pjoin(root, d), process_dir))
+            if root == process_dir:
+                # top level backend codes
+                dirs_to_make.append(os.path.relpath(pjoin(root, d), process_dir))
+            else:
+                # directory symlinks to create
+                store_dir_pth = pjoin(store_dir, os.path.relpath(pjoin(root, d), process_dir))
+                link_dir_pth = os.path.normpath(pjoin(root, os.readlink(pjoin(root, d))))
+                symlinks_to_make.append((link_dir_pth, store_dir_pth))
+
         for f in files:
             store_file_pth = pjoin(store_dir, os.path.relpath(pjoin(root, f), process_dir))
             link_file_pth = os.path.normpath(pjoin(root, os.readlink(pjoin(root, f))))
@@ -621,7 +655,11 @@ def move_process_data_to_store(repo_path: str, *, remote_operation: bool = False
         if not os.path.isdir(dpth):
             os.makedirs(dpth)
     for src, dest in symlinks_to_make:
-        symlink_rel(src, dest)
+        if os.path.isdir(src):
+            is_dir_link = True
+        else:
+            is_dir_link = False
+        symlink_rel(src, dest, is_dir=is_dir_link)
 
     # reset before releasing control.
     shutil.rmtree(process_dir)
@@ -629,7 +667,7 @@ def move_process_data_to_store(repo_path: str, *, remote_operation: bool = False
 
 
 def list_all_commits(refenv):
-    """returns a list of all commits stored in the repostiory
+    """returns a list of all commits stored in the repository
 
     Parameters
     ----------
@@ -647,7 +685,7 @@ def list_all_commits(refenv):
         with refTxn.cursor() as cursor:
             cursor.first()
             for k in cursor.iternext(keys=True, values=False):
-                commitKey = k.decode()[:40]
+                commitKey, *_ = k.decode().split(c.SEP_KEY)
                 commits.add(commitKey)
             cursor.close()
     finally:
