@@ -6,7 +6,10 @@ import string
 import weakref
 from io import StringIO
 from functools import partial
+from itertools import tee, filterfalse
 from typing import Union, Any
+import importlib
+import types
 
 import blosc
 import wrapt
@@ -15,6 +18,26 @@ from . import __version__
 from .constants import DIR_HANGAR
 
 
+class LazyImporter(types.ModuleType):
+    """Lazily import a module. `_load` adds the attributes of
+    importing module to `LazyLoader` instance. Hence `__getattr__` of `LazyLoader` will
+    be invoked only once. We might need to extend the `LazyLoader` class to have
+    functions like `__dir__` later.
+    """
+
+    def __init__(self, name: str):
+        super(LazyImporter, self).__init__(name)
+
+    def _import_module(self):
+        module = importlib.import_module(self.__name__)
+        self.__dict__.update(module.__dict__)
+        return module
+
+    def __getattr__(self, item):
+        module = self._import_module()  # this happens only once
+        return getattr(module, item)
+
+
 def set_blosc_nthreads() -> int:
     """set the blosc library to two less than the core count on the system.
 
@@ -26,8 +49,10 @@ def set_blosc_nthreads() -> int:
         ncores blosc will use on the system
     """
     nCores = blosc.detect_number_of_cores()
-    if nCores <= 2:
+    if nCores == 1:
         nUsed = 1
+    elif nCores == 2:
+        nUsed = 2
     elif nCores <= 4:
         nUsed = nCores - 1
     else:
@@ -53,9 +78,9 @@ def cm_weakref_obj_proxy(obj: Any) -> wrapt.ObjectProxy:
 
     This is required because (for some unknown reason) `weakproxy`
     references will not actually pass through the `__enter__` attribute of
-    the reffered object's instance. As such, these are manual set to the
+    the referred object's instance. As such, these are manually set to the
     appropriate values on the `weakproxy` object. The final `weakproxy` is
-    in turn referenced by a object proxy so that all calls to the
+    in turn referenced by an object proxy so that all calls to the
     methods/attributes are passed through uniformly.
 
     Parameters
@@ -76,7 +101,7 @@ def cm_weakref_obj_proxy(obj: Any) -> wrapt.ObjectProxy:
     return obj_proxy
 
 
-def symlink_rel(src: os.PathLike, dst: os.PathLike):
+def symlink_rel(src: os.PathLike, dst: os.PathLike, *, is_dir=False):
     """Create symbolic links which actually work like they should
 
     Parameters
@@ -85,18 +110,21 @@ def symlink_rel(src: os.PathLike, dst: os.PathLike):
         create a symbolic link pointic to src
     dst : os.PathLike
         create a link named dst
+    is_dir : bool, kwarg-only, optional
+        if pointing to a directory, set to true. Default = False
     """
     rel_path_src = os.path.relpath(src, os.path.dirname(dst))
-    os.symlink(rel_path_src, dst)
+    os.symlink(rel_path_src, dst, target_is_directory=is_dir)
 
 
-_SuitableCharRE = re.compile(r'[\w\.\-\_]+$', flags=re.ASCII)
+_SuitableCharRE = re.compile(r'[\w\.\-\_]+\Z', flags=re.ASCII)
 
 
 def is_suitable_user_key(key: Union[str, int]) -> bool:
-    """Checks if string contains only alpha-numeric ascii chars or ['.', '-' '_'] (no whitespace)
+    """Checks if only alpha-numeric ascii chars or ['.', '-' '_'] (no whitespace)
 
-    Necessary because python 3.6 does not have a str.isascii() method.
+    Necessary because python 3.6 does not have a str.isascii() method. In
+    addition, checks that all input keys are less than 64 characters long.
 
     Parameters
     ----------
@@ -115,6 +143,8 @@ def is_suitable_user_key(key: Union[str, int]) -> bool:
             str_data = str(key)
         else:
             raise TypeError
+        if len(str_data) > 64:
+            return False
         return bool(_SuitableCharRE.match(str_data))
     except TypeError:
         return False
@@ -137,11 +167,40 @@ def is_ascii(str_data: str) -> bool:
     """
     try:
         str_data.encode('ascii')
-    except UnicodeEncodeError:
+    except (UnicodeEncodeError, AttributeError):
         return False
     return True
 
 
+def pairwise(iterable):
+    "s -> (s0,s1), (s1,s2), (s2, s3), ..."
+    a, b = tee(iterable)
+    next(b, None)
+    return zip(a, b)
+
+
+def unique_everseen(iterable, key=None):
+    """List unique elements, preserving order. Remember all elements ever seen.
+
+    >>> list(unique_everseen('AAAABBBCCDAABBB'))
+    ['A', 'B', 'C', 'D']
+    >>> list(unique_everseen('ABBCcAD', str.lower))
+    ['A', 'B', 'C', 'D']
+    """
+    seen = set()
+    seen_add = seen.add
+    if key is None:
+        for element in filterfalse(seen.__contains__, iterable):
+            seen_add(element)
+            yield element
+    else:
+        for element in iterable:
+            k = key(element)
+            if k not in seen:
+                seen_add(k)
+                yield element
+
+
 def find_next_prime(N: int) -> int:
     """Find next prime >= N
 
@@ -175,7 +234,7 @@ def find_next_prime(N: int) -> int:
             return n
 
 
-def file_size(p: os.PathLike) -> int:
+def file_size(p: os.PathLike) -> int:  # pragma: no cover
     """Query the file size of a specific file
 
     Parameters
@@ -222,7 +281,7 @@ def folder_size(p: os.PathLike, *, recurse: bool = False) -> int:
     for entry in os.scandir(p):
         if entry.is_file(follow_symlinks=False):
             total += entry.stat().st_size
-        elif (recurse is True) and (entry.is_dir() is True):
+        elif (recurse is True) and (entry.is_dir(follow_symlinks=False) is True):
             total += folder_size(entry.path, recurse=True)
     return total
 
@@ -259,7 +318,7 @@ def is_valid_directory_path(p: os.PathLike) -> os.PathLike:
     if not os.path.isdir(usr_path):
         msg = f'Path arg `p`: {p} is not a directory.'
         raise NotADirectoryError(msg)
-    if not os.access(usr_path, os.W_OK):
+    if not os.access(usr_path, os.W_OK):  # pragma: no cover
         msg = f'User does not have permission to write to directory path: {p}'
         raise PermissionError(msg)
 
@@ -387,4 +446,4 @@ def readme_contents(user_name: str, user_email: str) -> StringIO:
     buf.write(f'Failure to do so will result in undesireable performance of version control\n')
     buf.write(f'systems meant for text/code such as Git, Mercurial, Subversion, etc.\n')
 
-    return buf
\ No newline at end of file
+    return buf
