@@ -10,3 +10,6 @@ import hangar.records
 print("import: 'hangar.remote'")
 import hangar.remote
 
+print("import: 'hangar.backends'")
+import hangar.backends
+
